@@ -1,4 +1,5 @@
 import os
+import pipes
 import signal
 
 
@@ -16,18 +17,24 @@ def exit_code_str(exception_name, command, exit_code):
     """
     str_ = "{} for command {}".format(exception_name, command)
     if exit_code is not None:
-        str_ += " (exited with code {:d})".format(exit_code)
-        if (os.name == "posix") and (exit_code > 128):
+        if exit_code < 0 or (os.name == "posix" and exit_code > 128):
             signals = {
                 number: name for name, number in vars(signal).items() if name.startswith("SIG")
             }
-            number = exit_code - 128
-            name = signals.get(number)
-            if name:
-                str_ += (
-                    "\nNote: this might indicate a fatal error signal "
-                    "({:d} - 128 = {:d}: {})".format(number + 128, number, name)
-                )
+            if exit_code < 0:
+                # Signal reported via subprocess.Popen.
+                sig_name = signals.get(-exit_code)
+                str_ += " (exited with code {:d} ({}))".format(exit_code, sig_name)
+            else:
+                str_ += " (exited with code {:d})".format(exit_code)
+                number = exit_code - 128
+                name = signals.get(number)
+                if name:
+                    str_ += (
+                        ")\nNote: this might indicate a fatal error signal "
+                        "({:d} - 128 = {:d}: {})".format(exit_code, number, name)
+                    )
+        str_ += " (exited with code {:d})".format(exit_code)
     return str_
 
 
@@ -42,12 +49,17 @@ class MissingSubstitution(Error):
 
     def __init__(self, name):
         self.name = name
+        super(Error, self).__init__(name)
 
 
 class ConfigError(Error):
     """Error in tox configuration."""
 
 
+class SubstitutionStackError(ConfigError, ValueError):
+    """Error in tox configuration recursive substitution."""
+
+
 class UnsupportedInterpreter(Error):
     """Signals an unsupported Interpreter."""
 
@@ -59,10 +71,11 @@ class InterpreterNotFound(Error):
 class InvocationError(Error):
     """An error while invoking a script."""
 
-    def __init__(self, command, exit_code=None):
+    def __init__(self, command, exit_code=None, out=None):
         super(Error, self).__init__(command, exit_code)
         self.command = command
         self.exit_code = exit_code
+        self.out = out
 
     def __str__(self):
         return exit_code_str(self.__class__.__name__, self.command, self.exit_code)
@@ -79,10 +92,12 @@ class MissingDependency(Error):
 class MissingRequirement(Error):
     """A requirement defined in :config:`require` is not met."""
 
+    def __init__(self, config):
+        self.config = config
 
-class MinVersionError(Error):
-    """The installed tox version is lower than requested minversion."""
+    def __str__(self):
+        return " ".join(pipes.quote(i) for i in self.config.requires)
 
-    def __init__(self, message):
-        self.message = message
-        super(MinVersionError, self).__init__(message)
+
+class BadRequirement(Error):
+    """A requirement defined in :config:`require` cannot be parsed."""
